open Lexing  (* distributed with OCaml *)
open Lexer
open Parser
open Pretty
open Ast


let dotty = ref false 
let parse = ref true
let file = ref ""

let speclist = [
  "-dot", Arg.Set dotty, "\tSet dotty output" ;
  "-noparse", Arg.Clear parse, "\tDisable parsing"
  ]

let anon_fun s =
  if !file = ""
  then file := s
  else failwith "Too many files"

let usage_msg =
  "\nUsage: " ^ Sys.argv.(0) ^ " [options] file.pas\n"


let main =

  Arg.parse speclist anon_fun usage_msg ;

  if !dotty && not !parse then
    failwith "You cannot choose -dot AND -noparse" ;

  let input = open_in_bin !file in
  let lexbuf = Lexing.from_channel input
  and p =
    {
     pos_fname = !file;
     pos_lnum = 1;
     pos_bol = 0;
     pos_cnum = 0;
    } in
  lexbuf.lex_curr_p <- p ;

  if !parse then

    let ast = Parser.main Lexer.token lexbuf in

    if !dotty then
      Ast.to_graph "out.dot" ast
    else
      print_endline (Ast.to_string ast)

  else

    while 
      let t = token lexbuf in
      print_token t ;
      t <> EOF
    do
      flush stdout
  done
